// How many Psycho Slings you can use per jump
local totalthoks = 3

local nonsillytaunts = 5
local tauntsoundtable = {
	sfx_s3k8b, // Pa-tonk! (falling over)
	sfx_s3k7b, // Earp. (shrug)
	sfx_lmao, // Ehehehe! (laughing)
	sfx_kc48, // Ba-da! (Gyaru peace sign)
	sfx_strpst, // Bi-ding! (thumbs up)
	sfx_ultins, // BWEH (drip)
	sfx_boingf // Boing! (get real)
}

// Initiate a Psycho Bounce
addHook("AbilitySpecial",function(p)
	if p.celadon
	and p.celadon.thoksleft
		p.celadon.thok = 1
		S_StartSound(p.mo,sfx_s3k74) // Hongg...!
	end
end)

// Spawn particles for using psychokinesis, including charging a Psycho Bounce and some regular animations
// Arguments:
// 	- mo: The (ideally player) object to spawn the aura effect from
//	- size: The size of the aura effect, from 0 for the size of the player to 35 for 1.25x the size of the player
//	- count: The transparency of the aura effect, from 0 to 64
// Yes, I know these ranges are stupid, but I don't feel like fixing them to work better for something that isn't a mess of spheres.
local pkParticles = function(mo,size,count)

	// Spawn a ghost... spooky!
	local ghost = P_SpawnGhostMobj(mo)
	if not (ghost and ghost.valid) return end
	
	// A whole bunch of properties
	ghost.color = SKINCOLOR_GREEN
	ghost.colorized = true
	ghost.scale = $+($*size/140)
	ghost.tics = 4
	ghost.frame = $&~ FF_TRANSMASK // Don't be transparent
	ghost.flags = $|MF_NOGRAVITY
	ghost.alpha = FU*count/64 // Actually, do be transparent, but correctly this time
	// Don't get separated from the player
	ghost.momx = mo.momx
	ghost.momy = mo.momy
	ghost.momz = mo.momz
	
end

// Hardcode Psycho Stomp interactions because coding is dumb and stupid
local shouldStomp = function(mo,p)
	// Extinguish flames
	if mo.flags & MF_FIRE
		P_KillMobj(mo)
		return false
		
	// Don't bounce at all
	elseif (mo.flags & (MF_SCENERY|MF_NOCLIP|MF_NOCLIPTHING|MF_SPRING)) // Anything with these flags...
	// ...or anything from this list of exceptions
	or mo.type == MT_STARPOST
	or mo.type == MT_SIGN
	or mo.type == MT_FLAMEHOLDER
	or mo.type == MT_STEAM
	or mo.type == MT_MINECART
	--or mo.type == MT_MINECARTSEG
	or mo.type == MT_MINECARTSPAWNER
		return false
	
	// Pushables: stay still if you're marked as unpushable, otherwise stay push
	elseif mo.info.flags & MF_PUSHABLE
		if mo.flags & MF_PUSHABLE
			P_InstaThrust(mo,R_PointToAngle2(p.mo.x,p.mo.y,mo.x,mo.y),20*p.mo.scale)
		end
		return false
	
	// Monitors: bounce straight up and break
	elseif (mo.flags & MF_MONITOR)
		P_SetObjectMomZ(mo,10*FU,false)
		P_KillMobj(mo,p.mo,p.mo)
		return false
	end
	
	return true
end

// Why is this still not built into Blua...?
local xor = function(a,b)
	if a and b
		return false
	end
	
	return a or b
end



// Main logic loop
addHook("PlayerThink",function(p)
	
	// Don't run Celadon logic if the player isn't Celadon
	local mo = p.mo
	if not (mo and mo.valid
	and mo.skin == "celadon")
		if p.celadon
			p.celadon = nil
		end
		return 
	end
	
	// Set up Celadon variable table
	if not p.celadon
		p.celadon = {
			thok = 0,				// How charged a Psycho Sling is, or 0 if you're not charging one
			thoksleft = 0,			// How many more Psycho Slings you can use before landing
			stomped = false,		// Whether or not you used a Psycho Stomp
			cuffopen = 0,			// How far open the cuff HUD graphic is, or 0 if it shouldn't be visible
			strongtimer = 0,		// Timer to remove pw_strong
			tauntsound = nil,		// The sound your taunt is making, or nil if you're not taunting
			starttaunt = false,		// Whether or not you were pressing Toss Flag last tic
			gokubombfuse = 0,		// How long until you explode, or 0 if you're not Goku
			minecarttaunt = 0,		// Which taunt you're doing in a minecart, or 0 if you're not taunting in a minecart
			waitbob = 0				// Angle for the sine motion of Celadon's waiting animation, or 0 if you're not waiting
		}
	end
	
	// Timer to remove PW_STRONG
	if p.celadon.strongtimer
		p.celadon.strongtimer = $-1
		if not p.celadon.strongtimer
			p.powers[pw_strong] = 0
		end
	end
	
	// Restore ability when on the ground or being carried
	if P_IsObjectOnGround(mo)
	or mo.eflags & MFE_JUSTHITFLOOR
	or p.powers[pw_carry]
		p.celadon.thoksleft = totalthoks
	end
	
	// Psycho Sling charging logic
	if p.celadon.thok
	and not p.exiting
	and not p.powers[pw_carry]
	and not P_PlayerInPain(p)
	and p.pflags & PF_JUMPED
	and p.playerstate == PST_LIVE
	
		// Start a Psycho Stomp
		if p.cmd.buttons & BT_SPIN
			mo.momx = $/2
			mo.momy = $/2
			P_SetObjectMomZ(mo,-40*FU,false)
			
			// Not charging a Psycho Sling, nor can you start one
			p.celadon.thok = 0
			p.celadon.thoksleft = 0
			
			p.celadon.stomped = true
			mo.state = S_PLAY_CELADON_STOMP
			
			p.powers[pw_strong] = STR_BUST|STR_SPIKE|STR_HEAVY|STR_STOMP
			S_StartSound(mo,sfx_kc4c) // Pfongf!
		
		// Keep charging
		elseif p.cmd.buttons & BT_JUMP
			// Slow descent
			mo.momz = $*9/10
			
			// If it'll be fully charged, play the sound
			if p.celadon.thok == 34
				S_StartSound(mo,sfx_cdfm23) // Ti-ting!
			end
			
			// Cap charge at 35
			p.celadon.thok = min(35,$+1)
			
			mo.state = S_PLAY_CELADONCHARGE
			
			//Direction indicator particles
			for i = 1,10
		
				local hamnt = 256 * sin((90*(p.celadon.thok * ANG1 / 35))) // How far the current dot should extend horizontally
				local vec = {
					x = FixedMul(hamnt,cos(mo.angle)),
					y = FixedMul(hamnt,sin(mo.angle)),
					z = 64 * cos((90*(p.celadon.thok * ANG1 / 35)))
				}
				
				// Adjust distance to match where we are in the loop
				vec.x = $*i/10
				vec.y = $*i/10
				vec.z = $*i/10
				
				// Flip with gravity
				if mo.eflags & MFE_VERTICALFLIP
					vec.z = -$
				end
				
				// Adjust to match scale
				vec.x = FixedMul($,mo.scale)
				vec.y = FixedMul($,mo.scale)
				vec.z = FixedMul($,mo.scale)
				
				// Center on Celadon vertically
				vec.z = $+(mo.height/2)
				
				// Spawn the dot, and give it some properties
				local thok = P_SpawnMobj(mo.x + vec.x, mo.y + vec.y, mo.z + vec.z, MT_THOK)
				thok.tics = 2
				thok.momx = mo.momx
				thok.momy = mo.momy
				thok.momz = mo.momz
				thok.frame = $ & FF_FRAMEMASK
				
				// Color and size the tip differently
				if i == 10
					thok.scale = mo.scale/3
					thok.color = skincolors[p.skincolor].invcolor
				else
					thok.scale = max(mo.scale/4,mo.scale*(15-i)/40)
					thok.color = p.skincolor
				end
			end
			
			pkParticles(mo,p.celadon.thok,48)
		
		// Let it rip!
		else
			// Cut speed beforehand to prevent infinite stacking
			mo.momx = $*2/3
			mo.momy = $*2/3
			mo.momz = $*2/3
			
			// Figure out the speed you should be going
			local speed = 8 + (8 * p.celadon.thoksleft / totalthoks)
			
			// Adjust speed based on environmental factors
			
			// No decay for Super Celadon
			if p.powers[pw_super]
				speed = 20
			end
			
			if mo.eflags & MFE_UNDERWATER
				speed = $*2/3
			end
			
			// Do a bunch of angle math to make sure the speed is right. Isn't trigonometry grand?
			P_Thrust(mo,mo.angle,3*speed * FixedMul(mo.scale,cos((90*(p.celadon.thok * ANG1 / 35)) - ANGLE_90)))
			P_SetObjectMomZ(mo,-speed * sin((90*(p.celadon.thok * ANG1 / 35)) - ANGLE_90),true)
			
			// A hard cap on top of the soft cap
			if p.speed >= 80*mo.scale
				P_InstaThrust(mo,R_PointToAngle2(0,0,mo.momx,mo.momy),80*mo.scale)
			end
			
			// Not charging a Psycho Sling anymore, and there's one less you can use
			p.celadon.thok = 0
			p.celadon.thoksleft = $-1
			
			// Different sound an animation depending on whether or not this is your last one
			if p.celadon.thoksleft
				mo.state = S_PLAY_ROLL
				S_StartSound(mo,sfx_cdfm62) // P-fyoom!
			else
				mo.state = S_PLAY_FALL
				S_StartSound(mo,sfx_s3k72) // Beong...
			end
			
			S_StartSound(mo,sfx_boingf) // Boing!
			p.powers[pw_strong] = STR_WALL|STR_CEILING|STR_SPIKE|STR_ATTACK
		end
		
	// Not using a Psycho Sling at all
	else
		p.celadon.thok = 0
		
		// Delay removing pw_strong
		if P_IsObjectOnGround(mo)
		and p.powers[pw_strong]
		and not p.celadon.strongtimer
			p.celadon.strongtimer = 2
		end
	end
	
	// Make Celadon shorter when ducking...
	if mo.state == S_PLAY_SPINDASH
		p.drawangle = mo.angle
		if not (mo.eflags & MFE_VERTICALFLIP) // Trying to be short in reverse gravity seldom goes well
			mo.height = P_GetPlayerSpinHeight(p)
		end
	// ...or not shorter when not ducking
	elseif not P_PlayerShouldUseSpinHeight(p)
		mo.height = P_GetPlayerHeight(p)
	end
	
	// Taunt in a minecart
	if p.cmd.buttons & BT_TOSSFLAG
	and p.powers[pw_carry] == CR_MINECART
	
		// Just starting a minecart taunt
		if not p.celadon.minecarttaunt
			// Decide which taunt to use
			local rand = P_RandomKey(nonsillytaunts)+1
			
			// Manually pick a taunt
			if p.cmd.buttons & BT_WEAPONMASK
			and p.cmd.buttons & BT_WEAPONMASK <= nonsillytaunts
				rand = p.cmd.buttons & BT_WEAPONMASK
			end
			
			// Play the sound, and keep track of it for later
			p.celadon.tauntsound = tauntsoundtable[rand]
			S_StartSound(mo,p.celadon.tauntsound)
			
			p.celadon.minecarttaunt = rand
		end
		
		// Swap to the taunt sprite, and jump to the appropriate frame
		mo.sprite2 = SPR2_MLEL
		mo.frame = ($&~FF_FRAMEMASK)|(p.celadon.minecarttaunt-1)
		
	// Ending a taunt
	elseif p.celadon.minecarttaunt
		if p.powers[pw_carry] == CR_MINECART
			mo.frame = ($&~FF_FRAMEMASK) // Stop the game from trying to use a nonexistent frame and then exploding about it
			mo.sprite2 = SPR2_STND
		end
		
		// Stop the sound your taunt made
		if p.celadon.tauntsound
			S_StopSoundByID(mo,p.celadon.tauntsound)
			p.celadon.tauntsound = nil
		end
		
		p.celadon.minecarttaunt = 0
	end
	
	// Spawn speed ghosts for Psycho Stomp
	if mo.state == S_PLAY_CELADON_STOMP
	and not P_IsObjectOnGround(mo)
		P_SpawnGhostMobj(mo)
		
	// If you stomped previously, but aren't stomping anymore
	elseif p.celadon.stomped
		p.celadon.stomped = false // Let the next frame know you stopped stomping
		
		// If you're on the ground, it's time for a Psycho Stomp landing
		if (P_IsObjectOnGround(mo)
		or (mo.eflags & MFE_JUSTHITFLOOR))
		and p.playerstate == PST_LIVE
		and mo.state != S_PLAY_SPRING
		and not P_PlayerInPain(p)
		
			S_StartSound(mo,sfx_pstop) // Thumwumwumwum!
			mo.state = S_PLAY_CELADON_LANDING // Landing animation with no speed cut. Take notes, Millie!
			// (Note: I also made BluEyes)
			
			local range = 200*mo.scale // How far away the shockwave will affect objects
			if p.powers[pw_super] // Super Celadon can send enemies farther
				range = $*2
			end
			
			// Now for the fun part: how the shockwave interacts with objects!
			searchBlockmap("objects",function(r,f)
				// Take note of the distance between Celadon and the found object - we'll be using it repeatedly
				local dist = R_PointToDist2(r.x,r.y,f.x,f.y)
				
				// Whether or not Celadon and the object are close enough vertically
				local sharedfloor = abs(r.z-f.z) <= range/4
				if r.eflags & MFE_VERTICALFLIP // The fact that you still have to check this like this is so stupid
					sharedfloor = abs((r.z+r.height)-(f.z+f.height)) <= range/4
				end
				
				// Common exit conditions
				if not (sharedfloor and dist <= range and f.health)
					return end
				
				// Break spikes and Egg Guard shields
				if (f.type == MT_SPIKE
				or f.type == MT_WALLSPIKE
				or f.type == MT_EGGSHIELD)
					P_KillMobj(f,r,r)
				
				// For everything else
				elseif P_IsObjectOnGround(f)
				and not (f.flags & MF_NOGRAVITY) // Can things without gravity be on the ground? Better safe than sorry, I suppose...
				and (r.eflags & MFE_VERTICALFLIP == f.eflags & MFE_VERTICALFLIP) // If you're on the floor and it's on the ceiling, that's not gonna work
				and shouldStomp(f,r.player) // Rule #2 of the moshpit: if they want out, let 'em out
					local weight = f.celadonweight
					if not weight
						weight = FU
					end
					
					if p.powers[pw_super]
						weight = $*2
					end
					
					local mult = FixedMul(mo.scale,FixedMul(weight,FixedDiv((range-dist),range)))
					
					P_SetObjectMomZ(f,10*mult,false) // Up, up...
					P_InstaThrust(f,R_PointToAngle2(r.x,r.y,f.x,f.y), 20*mult) // ...and away, at a speed proportional to one's proximity to the epicenter!
					
					// Hurt things if they're too close
					if dist <= range/2
					and mo.flags & MF_SHOOTABLE
						P_DamageMobj(f,r,r)
					end
				end
			end,mo,mo.x-range,mo.x+range,mo.y-range,mo.y+range)
			
			// Screen shake
			P_StartQuake(16*FU,8,{mo.x,mo.y,mo.z},512*mo.scale)
			
			// Spawn a shockwave
			for i = 0,32
				local dust = P_SpawnMobj(mo.x,mo.y,mo.z,MT_CELADUST)
				if dust and dust.valid
					// Disappears faster. move slower, and is smaller
					dust.fuse = 8
					dust.momz = $/2
					dust.scale = $/2
					
					P_InstaThrust(dust,i*ANG1/32*360,range/dust.fuse) // Travel the exact distance as a visual indicator of the range
				end
			end
		end
	end
	
	// Slightly hacky solution to make Mace Swings and Zoom Tubes still play spin sounds
	if p.powers[pw_carry] == CR_ZOOMTUBE
	or p.powers[pw_carry] == CR_MACESPIN
		if not p.celadonspinsound
			p.celadonspinsound = true
			S_StartSound(mo,sfx_s3k3c) // Play the Sonic 3 spin sound to bypass the skin sound check
			// ...Okay, fine, here's your funny onomatopoeia. Zjeeeng!
		end
	elseif p.celadonspinsound
		p.celadonspinsound = false
	end
	
	// Animate the cuff opening and closing when you can or can't use the Psycho Slinger
	if (p.pflags & PF_JUMPED)
	and not p.exiting
	and not p.powers[pw_carry]
	and not P_PlayerInPain(p)
		p.celadon.cuffopen = min(FU,$+(FU/8))
	else
		p.celadon.cuffopen = max(0,$-(FU/8))
	end
	
	// Super Celadon toggle (using code blatantly copied from Infinite)
	if p.powers[pw_super]
		p.charflags = $|SF_SUPER // Don't get interrupted by the game thinking you're not supposed to be super
		
	// Check to see if Celadon is using the custom bind to go super
	elseif p.cmd.buttons & BT_CUSTOM1 // Celadon goes super using Custom 1
	and not P_IsObjectOnGround(mo)
	and p.rings >= 50
	and All7Emeralds(emeralds)
	and (p.pflags & PF_JUMPED) // You have to have jumped, rather than bouncing off a spring or such
	and p.celadon.thoksleft == totalthoks // Haven't used your abilities yet
	
		p.charflags = $|SF_SUPER // Let the game know that Celadon is, in fact, supposed to be able to go super
		p.pflags = $&~ PF_JUMPED // Can't use abilities until you land
		P_DoSuperTransformation(p)
		
	else
		p.charflags = $&~ SF_SUPER // Disable goiung super the normal way
	end
end)

// Pre-think logic for Celadon, mostly input handling
addHook("PreThinkFrame",function()
	for p in players.iterate
		local mo = p.mo
		if not (mo and mo.valid
		and p.celadon)
			continue end
		
		// Make the crouch not act like a spindash, and prevent moving
		if mo.state == S_PLAY_SPINDASH
			if p.cmd.buttons & BT_SPIN
				p.pflags = $&~(PF_SPINNING|PF_APPLYAUTOBRAKE)
				p.cmd.forwardmove = 0
				p.cmd.sidemove = 0
			else // Cancel crouch
				mo.state = S_PLAY_STND
			end
		// Can't start sliding during the landing animation from a Psycho Stomp
		elseif mo.state == S_PLAY_CELADON_LANDING
			p.cmd.buttons = $&~ BT_SPIN
		end
		
		// If you're stomping but not falling, you're not really stomping, so stop trying
		if mo.state == S_PLAY_CELADON_STOMP
		and xor(mo.momz >= 0, mo.eflags & MFE_VERTICALFLIP) // One call in the whole script. Good use of a function, I'd say.
			mo.state = S_PLAY_FALL
		end
		
		if p.celadon.stomped
			p.pflags = $&~ PF_SPINDOWN
			p.cmd.buttons = $&~ BT_SPIN
		end
		
		// Handle an ongoing taunt
		if mo.state == S_PLAY_CELADON_TAUNT
			// Count down to exploding as Goku
			if p.celadon.gokubombfuse
				p.celadon.gokubombfuse = $-1
				
				// Esplode
				if p.celadon.gokubombfuse <= 0
					S_StopMusic(p)
					
					// Celadon can't be bothered to explode himself, so he just summons a bomb to do it for him
					local bomb = P_SpawnMobj(mo.x,mo.y,mo.z,MT_FBOMB)
					if bomb.valid	P_ExplodeMissile(bomb)	end
					
					// Die
					P_KillMobj(mo)
					
					// I tried to make him not fall, but it doesn't do anything. Probably because I did this in PreThinkFrame. Oops!
					mo.state = S_INVISIBLE
					mo.flags = $|MF_NOGRAVITY
					mo.momz = 0
				end
			end
			
			// Stop taunting...
			if ~p.cmd.buttons & BT_TOSSFLAG
				mo.state = S_PLAY_FALL
			// ...or don't, and just negate all input except continuing the taunt instead
			else
				p.pflags = $&~PF_APPLYAUTOBRAKE
				p.cmd.buttons = $ & (BT_TOSSFLAG|BT_WEAPONMASK)
				p.cmd.forwardmove = 0
				p.cmd.sidemove = 0
			end
		end
		
		// Not taunting...yet?
		if mo.state != S_PLAY_CELADON_TAUNT
		and not p.celadon.minecarttaunt
		
			// If you're not taunting, you're not Goku, so you don't have to worry about exploding
			p.celadon.gokubombfuse = 0
			
			// Start a taunt
			if p.cmd.buttons & BT_TOSSFLAG
			and p.celadon.starttaunt
			and not P_PlayerInPain(p)
			and not p.celadon.thok
			and mo.state != S_PLAY_CELADON_STOMP
			and not p.powers[pw_carry]
			and not (p.pflags & PF_SLIDING) // This fucker is why this has to be in PreThinkFrame
			and p.playerstate == PST_LIVE
				
				// 1 in 4 chance to add the rare taunts to the selectable pool
				local length = nonsillytaunts
				if P_RandomChance(FU/4)
					length = skins["celadon"].sprites[SPR2_MLEL].numframes
				end
				
				local rand = P_RandomKey(length)+1 // Decide which taunt to use
				
				// Manually pick a taunt
				if p.cmd.buttons & BT_WEAPONMASK
				and p.cmd.buttons & BT_WEAPONMASK <= nonsillytaunts
					rand = p.cmd.buttons & BT_WEAPONMASK
				end
				
				// Play the taunt's sound, and keep track of it for later
				p.celadon.tauntsound = tauntsoundtable[rand]
				S_StartSound(mo,p.celadon.tauntsound)
				
				// Set the state the appropriate amount of times to get the desired frame
				for i = 1,rand
					mo.state = S_PLAY_CELADON_TAUNT
				end
				
				// If you're Goku, get ready to explode
				if rand == 6
					p.celadon.gokubombfuse = 24
				end
				
			// Stop the sound a previous taunt made
			elseif p.celadon.tauntsound
				if S_SoundPlaying(mo,p.celadon.tauntsound)
					S_StopSoundByID(mo,p.celadon.tauntsound)
					p.celadon.tauntsound = nil
				end
			end
		end
		
		// Stop the taunt from re-triggering if a previous one gets interrupted by a state change
		p.celadon.starttaunt = (p.cmd.buttons & BT_TOSSFLAG == 0)
	end
end)

// Post think Celadon logic
addHook("PostThinkFrame",function()
	for p in players.iterate
		local mo = p.mo
		if not (mo and mo.valid
		and p.celadon)
			continue end
		
		// Sliding logic
		if p.pflags & PF_SPINNING
		and not p.powers[pw_carry]
		and mo.state != S_PLAY_SPINDASH
			p.powers[pw_strong] = STR_ANIM|STR_SPIKE // Break spikes until you get up
			
			// Sliding sound
			if P_IsObjectOnGround(mo)
			and not S_SoundPlaying(mo,sfx_celsld)
				S_StartSound(mo,sfx_celsld) // Hhhhhhhhhhhhh....
			end
			
			// Override a bunch of states
			if (mo.state == S_PLAY_ROLL
			or mo.state == S_PLAY_JUMP
			or mo.state == S_PLAY_WALK)
				mo.state = S_PLAY_CELADON_SLIDE
			end
		end
		
		// Stop the sliding sound if you're not sliding
		if not (p.pflags & PF_SPINNING and P_IsObjectOnGround(mo))
			S_StopSoundByID(mo,sfx_celsld)
		end
		
		// Make PK effects for animations that involve Celadon using psychokinesis (excluding SPR2_BNCE, since the Psycho Sling code handles that)
		if (mo.sprite2 == SPR2_WAIT // WAIT: lifting himself up to lounge in the air
		or mo.sprite2 == SPR2_EDGE // EDGE: pulling himself back off the ledge
		or mo.sprite2 == SPR2_RUN // RUN: pushing himself forward a little off the ground
		or mo.sprite2 == SPR2_SKID) // SKID: pushing back to slow down faster
			pkParticles(mo,17,16)
		end
		
		// Bob up and doing while waiting (detected via sprite2 for Anglestand compatibility)
		if mo.sprite2 == SPR2_WAIT
			if p.celadon.waitbob
				p.celadon.waitbob = ($+1) % 72 // Cycle of 72 frames, or just over 2 seconds
			else
				p.celadon.waitbob = 1 // This will both initialize it and properly increment it if it's at 0
			end
			
			mo.spriteyoffset = 2*sin(p.celadon.waitbob * ANG1 * 5) // Use sprite Y offset to give the illusion of hovering
			
		// Reset sprite Y offset only if it was just being used for Celadon's bobbing
		elseif p.celadon.waitbob
			p.celadon.waitbob = 0
			mo.spriteyoffset = 0
		end
		
		// Cancel the sliding animation if Celadon has stopped spinning
		if p.mo.state == S_PLAY_CELADON_SLIDE
		and not (p.pflags & PF_SPINNING)
			if P_IsObjectOnGround(p.mo)
				p.mo.state = S_PLAY_WALK
			else
				p.mo.state = S_PLAY_JUMP
			end
		end
	end
end)

// Let Celadon enter spin gaps, which makes him spin for some reason
addHook("PlayerCanEnterSpinGaps",function(p)
	if p.mo and p.mo.valid
	and p.celadon
	and p.mo.state == S_PLAY_SPINDASH
		return true
	end
end)

// Psycho Sling HUD
hud.add(function(v,p,cam)
	if p.celadon and p.celadon.cuffopen
		local dist = ease.linear(p.celadon.cuffopen,0,4*FU)
		
		v.drawFill(249,18,62,16,28|V_SNAPTORIGHT|V_SNAPTOTOP)
		v.drawFill(249,18,62*p.celadon.thoksleft/totalthoks,16,112|V_SNAPTORIGHT|V_SNAPTOTOP)
		v.drawScaled(248*FU,22*FU - dist,FU/4,v.cachePatch("CUFFTOP"),V_SNAPTOTOP|V_SNAPTORIGHT)
		v.drawScaled(248*FU,22*FU + dist,FU/4,v.cachePatch("CUFFBTM"),V_SNAPTOTOP|V_SNAPTORIGHT)
	end
end)

// Play Celadon's custom Super theme instead of the vanilla one
addHook("MusicChange", function(oldname, newname)
	if displayplayer
	and displayplayer.celadon
	and newname == "_super"
		return "celsup"
	end
end)